//
// JQUERY PLUGIN
// STYLED INPUT
// BY PAUL DIETRICH
//

// Enable Using Select Options by keyboard
var si_charstring = '';
var si_timestamp = 0;
var si_initialised = false;
function si_keydown_handler(e) {
	var options = $('.si-options:visible');
	if($(options).length == 1) {
		var code = e.keyCode || e.which;
		var character = String.fromCharCode(code).toLowerCase();
		var timestamp = new Date().getTime();
		if(si_timestamp < timestamp-1000) {
			si_charstring = '';
		}
		si_timestamp = timestamp;
		if(code != 16 && code != 20) {
			si_charstring += character;
		}
		if(code == 32) {	// Whitespace -> Prevent Scrolling Options Container
			return false;
		} else if(code == 40) {	// Down
			$(options).children('.si-selected').nextAll(':visible').first().mouseover().focus();
			return false;
		} else if(code == 38) {	// Up
			$(options).children('.si-selected').prevAll(':visible').first().mouseover().focus();
			return false;
		} else if(si_charstring) {
			$(options).children().each(function() {
				if($(this).html().substring(0,si_charstring.length).toLowerCase() == si_charstring && $(this).is(':visible')) {
					$(options).children().removeClass('si-selected');
					$(this).focus().addClass('si-selected');
					return false;
				}
			});
		}
	}
}

// Hide Select Options by click outside
function si_body_click_handler() {
	$('.si-options').hide();
}

(function ( $ ) {
	$.fn.styledInput = function(options,action) {
		options = options || {};
		action = action || '';
        var settings = $.extend({
            // These are the defaults.
            fileText : 'Datei auswählen',
			fileDeleteButton : false,
			
			fileDeleteButtonText : 'Auswahl entfernen',
			styleOptions : false
        }, options );
		var element = $(this);
		if(!action) {
			// Reset Inputs
			$(this).closest('form').bind('reset',function() {
				window.setTimeout(function() {
					$(element).styledInput(options,'reset');
				},10);
			});
			
			if(settings.styleOptions == true && si_initialised == false) {
				$('body').bind('click',si_body_click_handler);
				$(document).bind('keydown',si_keydown_handler);
			}
	
			$(this).each(function() {
				var type = (($(this).context.tagName == 'SELECT') ? 'select' : $(this).attr('type'));
				$(this).wrap('<div class="styledInput si-'+type+'"></div>');
				var input = $(this);
				var container = $(this).parent();
				$(container).addClass($(this).attr('class'));
				$(this).bind('change',function() {
					$(this).styledInput(options,'update');
				});
				$(container).bind('mouseover',function() {
					$(this).addClass('hover');
				});
				$(container).bind('mouseout',function() {
					$(this).removeClass('hover');
				});
				// SELECT OPTIONS
				if(type == 'select' && settings.styleOptions == true) {
					$(this).hide();
					$(container).append('<div class="si-options si-mouseout"></div>');
					var optionsbox = $(container).children('.si-options');
					$(this).children().each(function() {
						$(optionsbox).append('<a href="#" data-val="'+$(this).val()+'">'+$(this).html()+'</a>');
					});
					$(container).bind('click',function(e) {
						var isOpen = $(optionsbox).is(':visible');
						$('.si-options').hide();
						$('.si-select').removeClass('si-open');
						if(isOpen == false) {
							$(optionsbox).show();
							$(optionsbox).children().removeClass('si-selected');
							$(optionsbox).children('a[data-val="'+$(input).val()+'"]').addClass('si-selected');
							$(this).addClass('si-open');
						}
						e.stopPropagation();
					});
					$(optionsbox).bind('mouseover',function() {
						$(this).removeClass('si-mouseout');
					});
					$(optionsbox).bind('mouseout',function() {
						$(this).addClass('si-mouseout');
					});
					$(optionsbox).children('a').bind('mouseover',function() {
						$(optionsbox).children().removeClass('si-selected');
						$(this).addClass('si-selected');
					});
					$(optionsbox).children('a').bind('click',function(e) {
						if(!$(this).hasClass('hidden')) {
							if($(input).val() != $(this).attr('data-val')) {
								$(input).val($(this).attr('data-val'));
								$(input).styledInput(options,'update');
								$(input).change();
							}
							$(optionsbox).hide();
						}
						e.stopPropagation();
						return false;
					});
				}
				// FILE
				if(type == 'file') {
					if($(this).data('path') && $(this).data('filename')) {
						$(container).append('<input type="hidden" name="'+$(this).attr('name')+'_path" value="'+$(this).data('path')+'">');
						$(container).append('<input type="hidden" name="'+$(this).attr('name')+'_filename" value="'+$(this).data('filename')+'">');
						$(this).attr('title',$(this).data('filename'));
					}
					if(settings.fileDeleteButton == true) {
						$(container).append('<a class="si-file-delete" href="#" title="'+settings.fileDeleteButtonText+'" style="display:none"></a>');
						$(container).children('.si-file-delete').bind('click',function() {
							var fileinput = $(container).children('input[type="file"]');
							$(container).children('input[type="hidden"]').remove();
							fileinput.wrap('<form>').parent('form').trigger('reset').trigger('reset');	// Firing twice is nessesary cause of stupid IE...
							fileinput.unwrap();
							fileinput.data('filename','');
							fileinput.attr('title','');
							fileinput.attr('value','');
							fileinput.styledInput(options,'update');
							return false;
						});
					}
				}
				// SELECT & FILE
				if(type == 'select' || type == 'file') {
					$(container).append('<span></span>');
					$(this).css('height',$(container).css('height'));
				}
				// INHERIT ELEMENT ID
				if($(this).attr('id')) {
					$(container).attr('id','si-'+($(this).attr('id')));
				}
				$(this).styledInput(options,'update');
			});
			si_initialised = true;
		} else {
			$(this).each(function() {
				var type = (($(this).context.tagName == 'SELECT') ? 'select' : $(this).attr('type'));
				var value = $(this).val();
				var container = $(this).parent();
				// CHECKBOX & RADIO
				if(type == 'checkbox' || type == 'radio') {
					if($(this).is(':checked')) {
						$(container).removeClass('empty').addClass('checked');
					} else {
						$(container).removeClass('checked').addClass('empty');
					}
				}
				// UPDATE ALL OTHER RADIO INPUTS
				if(type == 'radio' && action == 'update') {
					$('input[name="'+$(this).attr('name')+'"]').styledInput(options,'updateradio');
				}
				// SELECT
				if(type == 'select') {
					if($(this).data('label')) {
						$(container).children('span').html($(this).data('label'));
					} else {
						$(container).children('span').html($(this).children('[value="'+value+'"]').text()+'&nbsp;');
					}
					$(container).find('.si-options a').removeClass('si-selected');
					$(container).find('.si-options a[data-val="'+value+'"]').addClass('si-selected');
				}
				// FILE
				if(type == 'file') {
					if(value) {
						$(container).children('input[type="hidden"]').remove();
					}
					var filename = value || $(this).data('filename');
					if(filename) {
						$(container).children('a.si-file-delete').show();
						filename = filename.substring(filename.lastIndexOf('\\')+1);
					} else {
						$(container).children('a.si-file-delete').hide();
					}
					value = filename || settings.fileText;
					$(container).children('span').html(value+'&nbsp;');
				}
			});
		}
	};
}( jQuery ));